package com.kuyuntech.cofarcooking.device.sdk.util.core;

import com.kuyuntech.cofarcooking.device.sdk.constant.core.*;
import com.kuyuntech.cofarcooking.device.sdk.devmode.airfryer.*;
import com.kuyuntech.cofarcooking.device.sdk.devmode.core.*;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.core.DevInfo;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.event.*;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.listener.Dev2McuDevEventListener;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.BaseBody;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.cmd.*;
import lombok.Data;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.greenrobot.eventbus.EventBus;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 昌久料理机SDK
 */
@Data
@ToString
public class CofarSDK {

    private static String BRAND_NUM = "";

    //设备状态
    private static DevInfo devInfo = DevInfo.builder().build();

    private static EventBus eventBus = EventBus.getDefault();

    private static boolean init = false;

    private static boolean dev = false;

    private static Timer timer;

    private static Long lastDevInfoTime = 0L;

    private static boolean recDevInfo = true;

    private static String runningInstId = "";

    private static String settingInstId = "";

    private static DevInfo settingDevInfo = DevInfo.builder().build();

    private CofarSDK() {

    }

    public static void initDevTest() {
        dev = true;
        //模拟心跳
        new Timer().schedule(new TimerTask() {
            @Override
            public void run() {
                updateDevInfo(devInfo());
            }
        }, 0, 1000);

        register(new Dev2McuDevEventListener(event -> {


            try {


                if (event.getBaseBody() instanceof StartCmd) {
                    devInfo().setStatus(DevStatus.RUNNING);
                    devInfo().setRunningStatus(DevStatus.RUNNING);
                    updateDevInfo(devInfo());
                }

                if (event.getBaseBody() instanceof PauseCmd) {
                    devInfo().setStatus(DevStatus.PAUSE);
                    devInfo().setRunningStatus(DevStatus.PAUSE);
                    updateDevInfo(devInfo());
                }

                if (event.getBaseBody() instanceof StopCmd) {
                    devInfo().setStatus(DevStatus.STOP);
                    //devInfo().setRemainTime((short) 0);
                    devInfo().setRunningStatus(DevStatus.STOP);
                    updateDevInfo(devInfo());
                }


                if (event.getBaseBody() instanceof CfgHeatCmd) {
                    CfgHeatCmd cfgHeatCmd = new CfgHeatCmd(event.getBaseBody());
                    devInfo().setTargetTemp(Converter.hexToShort(cfgHeatCmd.getTemp()));
                    devInfo().setTemp((short) (devInfo().getTargetTemp() - 1));
                    updateDevInfo(devInfo);
                }


                if (event.getBaseBody() instanceof CfgMotorCmd) {
                    CfgMotorCmd cfgMotorCmd = new CfgMotorCmd(event.getBaseBody());
                    devInfo().setMotorDirection(Converter.hexToByte(cfgMotorCmd.getDirection()));
                    devInfo().setMotorGear(Converter.hexToByte(cfgMotorCmd.getGear()));
                    updateDevInfo(devInfo);
                }

                if (event.getBaseBody() instanceof CfgTimeCmd) {
                    CfgTimeCmd cfgTimeCmd = new CfgTimeCmd(event.getBaseBody());
                    devInfo().setTargetTime(Converter.hexToShort(cfgTimeCmd.getTime()));
                    updateDevInfo(devInfo);
                }

                if (event.getBaseBody() instanceof WeightStartCmd) {
                    devInfo().setWeightStatus(true);

                    new Timer().schedule(new TimerTask() {
                        @Override
                        public void run() {
                            devInfo().setWeight((short) (Math.random() * 5000));
                            updateDevInfo(devInfo());
                            if (!devInfo().isWeightStatus()) {
                                cancel();
                            }
                        }
                    }, 0, 1000);
                    updateDevInfo(devInfo());
                }

                if (event.getBaseBody() instanceof WeightStopCmd) {
                    devInfo().setWeightStatus(false);
                    updateDevInfo(devInfo());
                }


            } catch (Exception e) {
                e.printStackTrace();
            }


        }));
    }

    /**
     * 蜂鸣器设置
     *
     * @param type  鸣响类型
     * @param count 鸣响次数
     */
    public static void cfgBeep(byte type, byte count) {
        CfgBeepCmd cmd = new CfgBeepCmd(type, count);
        post2Mcu(cmd);
    }

    /**
     * 电动
     */
    public static void jog() {
        JogCmd cmd = new JogCmd();
        post2Mcu(cmd);
    }


    /**
     * 切换模式
     *
     * @param mode
     * @param id
     */
    public static void changeModeWithoutCfg(String id, BaseDevMode mode) {

        if (DevStatus.STOP != devInfo().getStatus()) {
            settingInstId = id;
            settingDevInfo.setMode(mode);
            updateDevInfo(devInfo);
            return;
        }

        settingInstId = id;
        runningInstId = id;
        devInfo().setMode(mode);
        updateDevInfo(devInfo);
    }



    private static long lockTime = 0;

    /**
     * 切换模式
     *
     * @param mode
     * @param id
     */
    public static void changeMode(String id, BaseDevMode mode) {

        lockTime = new Date().getTime();




        if (DevStatus.STOP != devInfo().getStatus()) {
            settingInstId = id;
            settingDevInfo.setMode(mode);
            cfgHeat((short) 0, mode.getDefaultHeatMode());
            cfgMotor((byte) mode.getDefaultMotorGear(), (byte) mode.getMotorDirection(), (byte) 0x01);
            cfgHeat((short) mode.getDefaultTemp(), mode.getDefaultHeatMode());
            cfgTime((short) mode.getDefaultTime());
//            updateDevInfo(devInfo);
            return;
        }

        settingInstId = id;
        runningInstId = id;
        devInfo().setMode(mode);
        cfgHeat((short) 0, mode.getDefaultHeatMode());
        cfgMotor((byte) mode.getDefaultMotorGear(), (byte) mode.getMotorDirection(), (byte) 0x01);
        cfgHeat((short) mode.getDefaultTemp(), mode.getDefaultHeatMode());
        cfgTime((short) mode.getDefaultTime());
//        updateDevInfo(devInfo);
    }

    /**
     * 切换模式
     *
     * @param mode
     * @param id
     */
    public static DevInfo changeMode(String id, BaseDevMode mode, boolean restore) {


        if (!restore || !configRecord.containsKey(mode.getClass().getName())) {
            changeMode(id, mode);
        } else { //恢复记录参数
            changeModeWithoutCfg(id, mode);
            DevInfo param = configRecord.get(mode.getClass().getName());
            cfgMotor((byte) param.getMotorGear(), (byte) param.getMotorDirection(), (byte) 0x01);
            cfgHeat((short) param.getTargetTemp(), mode.getDefaultHeatMode());
            cfgTime((short) param.getTargetTime());
            updateDevInfo(devInfo);
            return param;
        }

        return null;

    }

    public static void resetLastPotStatus(){
         devInfo.setLastPotStatus(PotStatus.EXIST);
    }

    /**
     * 切换模式
     *
     * @param mode
     */
    public static void changeMode(BaseDevMode mode) {
        if (DevStatus.STOP != devInfo().getStatus()) {
            return;
        }
        devInfo().setMode(mode);
        cfgHeat((short)0, mode.getDefaultHeatMode());

        cfgMotor((byte) mode.getDefaultMotorGear(), (byte) mode.getMotorDirection(), (byte) 0x01);

        cfgHeat((short) mode.getDefaultTemp(), mode.getDefaultHeatMode());
        cfgTime((short) mode.getDefaultTime());
//        updateDevInfo(devInfo);
    }


    /**
     * 加热设置
     *
     * @param temp 温度
     * @param mode 加热模式
     */
    public static void cfgHeat(short temp, byte mode) {

        temp = parseTemp(temp);

        if (!settingInstId.equals(runningInstId)) {
            settingDevInfo.setTargetTemp(temp);
            settingDevInfo.setHeatMode(mode);
            updateDevInfo(devInfo);
            return;
        }

        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setTargetTempBuffer(temp);
            return;
        }

        cfgHeatNative(temp, mode);
    }

    /**
     * 加热设置
     *
     * @param temp 温度
     * @param mode 加热模式
     */
    public static void cfgHeatNative(short temp, byte mode) {

        temp = parseTemp(temp);

        CfgHeatCmd cmd = new CfgHeatCmd(temp, mode);
        post2Mcu(cmd);
    }

    /**
     * 电机档位
     *
     * @param gear        电机档位，正转设置范围0-15，反转范DevCommonEvent围0-3
     * @param direction   电机转向，0x00正转 0x01反转
     * @param highetSpeed 电机高低速选择，默认 0x01高速
     */
    public static void cfgMotor(byte gear, byte direction, byte highetSpeed) {

        if (!settingInstId.equals(runningInstId)) {
            settingDevInfo.setMotorGear(gear);
            settingDevInfo.setMotorDirection(direction);
            updateDevInfo(devInfo);
            return;
        }


        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setMotorGearBuffer(gear);
            devInfo().setMotorDirectionBuffer(direction);
            return;
        }

        cfgMotorNative(gear, direction, highetSpeed);
    }

    /**
     * 电机档位
     *
     * @param gear        电机档位，正转设置范围0-15，反转范围0-3
     * @param direction   电机转向，0x00正转 0x01反转
     * @param highetSpeed 电机高低速选择，默认 0x01高速
     */
    public static void cfgMotorNative(byte gear, byte direction, byte highetSpeed) {


        CfgMotorCmd cmd = new CfgMotorCmd(direction, gear, highetSpeed);
        post2Mcu(cmd);
    }

    /**
     * 设置点击档位
     *
     * @param gear 档位
     */
    public static void cfgMotorGear(byte gear) {

        if (!settingInstId.equals(runningInstId)) {
            settingDevInfo.setMotorGear((byte) settingDevInfo.getMode().checkMotorGear(gear));
            updateDevInfo(devInfo);
            return;
        }

        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setMotorGearBuffer((byte) devInfo().getMode().checkMotorGear(gear));
            return;
        }

        cfgMotorGearNative((byte) devInfo().getMode().checkMotorGear(gear));
    }

    /**
     * 设置电机档位
     *
     * @param gear 档位
     */
    public static void cfgMotorGearNative(byte gear) {

        CfgMotorCmd cmd = new CfgMotorCmd(devInfo.getMotorDirectionBuffer() != -1 ? devInfo.getMotorDirectionBuffer() : devInfo().getMotorDirection(), gear, (byte) 0x01);
        post2Mcu(cmd);
    }

    /**
     * 设置电机方向
     *
     * @param direction 电机转向，0x00正转 0x01反转
     */
    public static void cfgMotorDirection(byte direction) {

        if (!settingInstId.equals(runningInstId)) {
            settingDevInfo.setMotorDirectionBuffer((byte) direction);
            updateDevInfo(devInfo);
            return;
        }

        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setMotorDirectionBuffer(direction);
            updateDevInfo(devInfo);
            return;
        }

        cfgMotorDirectionNative(direction);
    }

    /**
     * 设置电机方向
     *
     * @param direction 电机转向，0x00正转 0x01反转
     */
    public static void cfgMotorDirectionNative(byte direction) {


        CfgMotorCmd cmd = new CfgMotorCmd(direction, devInfo().getMotorGear(), (byte) 0x01);
        post2Mcu(cmd);
    }

    /**
     * 时间设置
     *
     * @param time 时间，单位秒
     */
    public static void cfgTime(int time) {

        if (!settingInstId.equals(runningInstId)) {
            settingDevInfo.setTargetTime(settingDevInfo.getMode().checkTime(time));
            updateDevInfo(devInfo);
            return;
        }

        BaseDevMode mode = devInfo().getMode();

        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setTargetTimeBuffer(mode.checkTime(time));
            return;
        }


        cfgTimeNatvie(mode.checkTime(time));
    }

    /**
     * 设置喷水档位
     *
     * @param waterBear 喷水档位
     */
    public static void cfgWaterBear(int waterBear) {

        if (!settingInstId.equals(runningInstId)) {
            devInfo().setWaterGear(devInfo().getMode().checkWaterGear(waterBear));
            updateDevInfo(devInfo);
            return;
        }

        BaseDevMode mode = devInfo().getMode();

        if (DevStatus.RUNNING == devInfo().getStatus() || DevStatus.PAUSE == devInfo().getStatus()) {
            devInfo().setWaterGearBuffer(mode.checkWaterGear(waterBear));
            return;
        }

        devInfo().setWaterGear(devInfo().getMode().checkWaterGear(waterBear));
        updateDevInfo(devInfo());

    }


    /**
     * 时间设置
     *
     * @param time 时间，单位秒
     */
    public static void cfgTimeNatvie(int time) {
        devInfo().setTargetTime(time);
        updateDevInfo(devInfo());

    }

    public static void startCountDown() {
        devInfo().setRemainTime(devInfo().getTargetTime());
        countDown();
    }

    private static void countDown() {
        if (timer != null) {
            timer.cancel();
        }

        timer = new Timer();
        timer.schedule(new TimerTask() {

            @Override
            public void run() {

                if (devInfo().getMode().isDiyCountDown()) {


                    if (DevStatus.STOP == devInfo().getStatus() && (!(devInfo().getMode() instanceof Turbo) && !(devInfo().getMode() instanceof Knead))) {
                        CofarSDK.stop(false);
                        this.cancel();
                        return;
                    }



                    devInfo().getMode().onCountDown(this);
                } else {

                    if (!devInfo().getMode().startCountTime()) {
                        return;
                    }

                    if (devInfo().getMode().canStop()) {
                        CofarSDK.stop();
                        this.cancel();
                        return;
                    }

                    if ((devInfo().getRemainTime() <= 0 && devInfo().getTargetTime() != 0 && devInfo().getMode().isCountDown()) || (devInfo().getRemainTime() > devInfo().getMode().getMaxTime())) {
                        CofarSDK.stop(devInfo().getRunningRecipeId() == null);
                        this.cancel();
                        return;
                    }


                    if (devInfo.getMotorGear() >= 7 && (!devInfo.getMode().isCountDown() || devInfo().getTargetTime() == 0) && devInfo().getRemainTime() > (10 * 60 + lastReaintime)) {
                        CofarSDK.stop(devInfo().getRunningRecipeId() == null);
                        this.cancel();
                        return;
                    }


//                    if (DevStatus.PAUSE == devInfo().getStatus()) {
//                        CofarSDK.stop(false);
//                        this.cancel();
//                        return;
//                    }

                    if (DevStatus.STOP == devInfo().getStatus()) {
                        CofarSDK.stop(false);
                        this.cancel();
                        return;
                    }

                    if (DevStatus.RUNNING == devInfo().getStatus()) {
                        if (devInfo().getMode().isCountDown() && devInfo().getTargetTime() != 0) {
                            devInfo().setRemainTime((devInfo().getRemainTime() - 1));
                        } else {
                            devInfo().setRemainTime((devInfo().getRemainTime() + 1));
                        }

                    }

                }


            }
        }, 1000, 1000);
    }


    private static int lastReaintime = 0;

    public static void confirmConfig() {
        lastReaintime = devInfo().getRemainTime();
        if (devInfo().getTargetTimeBuffer() != -1) {
            cfgTimeNatvie(devInfo().getTargetTimeBuffer());
            devInfo().setRemainTime(devInfo().getTargetTimeBuffer());
        }
        if (devInfo().getMotorDirectionBuffer() != -1) cfgMotorDirectionNative(devInfo().getMotorDirectionBuffer());
        if (devInfo().getMotorGearBuffer() != -1) cfgMotorGearNative(devInfo().getMotorGearBuffer());
        if (devInfo().getTargetTempBuffer() != -1)
            cfgHeatNative(devInfo().getTargetTempBuffer(), devInfo().getHeatMode());
        devInfo().setTargetTempBuffer((short) -1);
        devInfo().setMotorGearBuffer((byte) -1);
        devInfo().setMotorDirectionBuffer((byte) -1);
        devInfo().setTargetTimeBuffer((byte) -1);
    }

    /**
     * 确认参数
     */
    public static void confirm() {


        //未设置参数不能运行
        if (devInfo().getTemp() >= 60 && devInfo().getMotorGearBuffer() > 3) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .msg("hight_temp_warning_tips")
                    .confirmButtonText("yes")
                    .cancelBtnText("no")
                    .showCancelBtn(true)
                    .showConfirmBtn(true)
                    .confirm(CofarSDK::confirmConfig)
                    .build();
            eventBus.post(devPromptEvent);
        } else {
            confirmConfig();
        }


//        if (devInfo().getTargetTimeBuffer() != -1) {
//            cfgTimeNatvie(devInfo().getTargetTimeBuffer());
//            devInfo().setRemainTime(devInfo().getTargetTimeBuffer());
//        }
//        if (devInfo().getMotorDirectionBuffer() != -1) cfgMotorDirectionNative(devInfo().getMotorDirectionBuffer());
//        if (devInfo().getMotorGearBuffer() != -1) cfgMotorGearNative(devInfo().getMotorGearBuffer());
//        if (devInfo().getTargetTempBuffer() != -1)
//            cfgHeatNative(devInfo().getTargetTempBuffer(), devInfo().getHeatMode());
//        devInfo().setTargetTempBuffer((short) -1);
//        devInfo().setMotorGearBuffer((byte) -1);
//        devInfo().setMotorDirectionBuffer((byte) -1);
//        devInfo().setTargetTimeBuffer((byte) -1);
    }

    /**
     * 取消参数改变
     */
    public static void cancel() {
        devInfo().setTargetTempBuffer((short) -1);
        devInfo().setMotorGearBuffer((byte) -1);
        devInfo().setMotorDirectionBuffer((byte) -1);
        devInfo().setTargetTimeBuffer((byte) -1);
    }

    /**
     * 开始运行菜谱
     *
     * @param recipeId 菜谱id
     * @param ext      拓展数据
     */
    public static void startRecipe(String recipeId, String ext) {
        devInfo().setRunningRecipeId(recipeId);
        devInfo().setRunningRecipeExt(ext);
    }


    /**
     * 开始运行菜谱
     *
     * @param recipeId
     */
    public static void startRecipe(String recipeId) {
        devInfo().setRunningRecipeId(recipeId);
        devInfo().setRunningRecipeExt("");
    }

    /**
     * 停止菜谱
     */
    public static void stopRecipe() {
        devInfo().setRunningRecipeId("");
        devInfo().setRunningRecipeExt("");
    }


    /**
     * 启动设备
     */
    public static void startRunning(String id) {
        if (settingInstId.equals(runningInstId)) {
            return;
        }
        runningInstId = settingInstId;
        devInfo().getMode().stop(false);
        devInfo().setStatus(DevStatus.STOP);
        changeMode(settingDevInfo.getMode());
        cfgMotorNative(settingDevInfo.getMotorGear(), settingDevInfo.getMotorDirection(), (byte) 1);
        cfgHeatNative(settingDevInfo.getTargetTemp(), settingDevInfo.getHeatMode());
        cfgTimeNatvie(settingDevInfo.getTargetTime());
        start(id);
    }

   public static Map<String,Object> startConfig = new HashMap<>();

    public static void startWithConfig(String id,Map<String,Object> config){
        startConfig = config;
        start(id);
    }

    /**
     * 启动设备
     */
    public static void start(String id) {


        postErrorTips(devInfo.getErrCode(), true);

        if (devInfo().getErrCode() != 0) {
            return;
        }

        //未设置参数不能运行
        if (!((DevModes.SOUP.equals(devInfo().getDevMode()) && devInfo().getPotCloverType() == PotTypes.SOUP) || (DevModes.AIR_FRYER.equals(devInfo().getDevMode()) && devInfo().getPotCloverType() == PotTypes.AIR_FRYER))) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("error_pot_clover_tips")
                    .confirm(() -> {
                        if (devInfo().getPotCloverType() == PotTypes.SOUP) {
                            changeDevModeFocus(DevModes.SOUP);
                        }

                        if (devInfo().getPotCloverType() == PotTypes.AIR_FRYER) {
                            changeDevModeFocus(DevModes.AIR_FRYER);
                        }

                    })
                    .build();
            eventBus.post(devPromptEvent);
            startConfig = new HashMap<>();
            return;
        }

        //未设置参数不能运行
        if (devInfo().getTargetTemp() == 0 && devInfo().getMotorGear() == 0 && !(devInfo.getMode() instanceof Turbo)) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("unset_param_tips")
                    .build();
            eventBus.post(devPromptEvent);
            startConfig = new HashMap<>();

            return;
        }

        //Turbo高温不允许运行
        if (devInfo.getMode() instanceof Turbo && CofarSDK.devInfo().getTemp() >= 60) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("hight_temp_turbo_tips")
                    .build();
            CofarSDK.promptUp(devPromptEvent);
            startConfig = new HashMap<>();

            return;
        }


        //不支持正计时提示设置时间参数
        if (!devInfo.getMode().isSupportCountUp() && CofarSDK.devInfo().getTargetTime() <= 0) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("unset_target_time")
                    .build();
            CofarSDK.promptUp(devPromptEvent);
            startConfig = new HashMap<>();

            return;
        }

        //未设置参数不能运行
        if (devInfo().getTemp() >= 60 && devInfo().getMotorGear() > 3) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .msg("hight_temp_warning_tips")
                    .showCancelBtn(true)
                    .showConfirmBtn(true)
                    .confirmButtonText("yes")
                    .cancelBtnText("no")
                    .confirm(() -> devInfo().getMode().start())
                    .build();
            eventBus.post(devPromptEvent);
            startConfig = new HashMap<>();

            return;
        }

        //未设置参数不能运行
        if (devInfo().getPotCloverStatus() == 1) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .msg("pot_clover_not_clost_tips")
                    .showCancelBtn(false)
                    .build();
            eventBus.post(devPromptEvent);
            startConfig = new HashMap<>();

            return;
        }
        lastErrorTips = -1;
        devInfo().setRunningInstId(id);
        devInfo().getMode().start();
    }

    public static void startNative() {


        if (devInfo().getMode() instanceof Steam) {
            if (devInfo().getTargetTemp() < 110) {
                cfgHeat((short) 100, devInfo().getMode().getDefaultHeatMode());
            }

            if (devInfo().getTargetTemp() >= 110 && devInfo().getTargetTemp() < 120) {
                cfgHeat((short) 110, devInfo().getMode().getDefaultHeatMode());
            }
        }

        recDevInfo = false;
        devInfo().setRemainTime(devInfo().getTargetTime());
        StartCmd cmd = new StartCmd();
        post2Mcu(cmd);
        devInfo().setStatus(DevStatus.RUNNING);
        updateDevInfo(devInfo());


        ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
        //开始计时
        executorService.schedule(CofarSDK::countDown, 1, TimeUnit.SECONDS);


    }

    /**
     * 恢复程序
     */
    public static void resume() {

        //未设置参数不能运行
        if (devInfo().getTemp() >= 60 && devInfo().getMotorGear() > 3) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .msg("hight_temp_warning_tips")
                    .confirmButtonText("yes")
                    .cancelBtnText("no")
                    .showCancelBtn(true)
                    .showConfirmBtn(true)
                    .confirm(() -> devInfo().getMode().resume())
                    .build();
            eventBus.post(devPromptEvent);
        } else {


            postErrorTips(devInfo.getErrCode(), true);

            if (devInfo().getErrCode() != 0) {
                return;
            }

            //未设置参数不能运行
            if (devInfo().getPotCloverStatus() == 1) {
                DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                        .msg("pot_clover_not_clost_tips")
                        .showCancelBtn(false)
                        .build();
                eventBus.post(devPromptEvent);
                return;
            }

            devInfo().getMode().resume();
        }


    }

    /**
     * 恢复程序
     */
    public static void resumeNavtive() {


        recDevInfo = false;
        StartCmd cmd = new StartCmd();
        post2Mcu(cmd);
        devInfo().setStatus(DevStatus.RUNNING);
        updateDevInfo(devInfo());
        countDown();
    }

    /**
     * 暂停设备
     */
    public static void pause() {
        devInfo().getMode().pause();
    }

    /**
     * 暂停设备
     */
    public static void pauseNatvie() {
        if (timer != null && !devInfo().getMode().isDiyCountDown()) timer.cancel();
        devInfo.setStatus(DevStatus.PAUSE);
        updateDevInfo(devInfo());
//        StopCmd cmd = new StopCmd();
//        post2Mcu(cmd);
        PauseCmd cmd = new PauseCmd();
        post2Mcu(cmd);

    }

    /**
     * 停止设备
     */
    public static void stop() {

        stop(true);
    }

    /**
     * 停止设备,无提示
     */
    public static void stop(boolean tips) {
        if(startConfig != null && startConfig.containsKey("stopTips")){
            tips = (boolean)startConfig.get("stopTips");
        }
        devInfo().getMode().stop(tips );
    }

    /**
     * 停止设备
     */
    public static void stopNative() {
        lastReaintime = 0;
        if (timer != null && !devInfo().getMode().isDiyCountDown()) timer.cancel();
        StopCmd cmd = new StopCmd();
        post2Mcu(cmd);
        devInfo().setStatus(DevStatus.STOP);
        if (!devInfo().getMode().isCountDown()) {
            devInfo().setRemainTime(devInfo().getTargetTime());
        } else {
            devInfo().setRemainTime((short) 0);
        }
        devInfo.setMotorGearBuffer((byte) -1);
//        devInfo.setMotorGear((byte)-1);
        devInfo.setTargetTimeBuffer((byte) -1);
        devInfo.setTargetTempBuffer((byte) -1);
        devInfo().setArrive37Temp(false);
        updateDevInfo(devInfo());
    }

    /**
     * 设备暂停
     */
    public static void superStop() {
        stop();
    }

    /**
     * 查询设备信息
     */
    public static void versionQuery() {
        VersionQueryCmd cmd = new VersionQueryCmd();
        post2Mcu(cmd);
    }

    /**
     * 开始称重校准
     */
    public static void startWeightAlign() {
        WeightAlignStartCmd cmd = new WeightAlignStartCmd();
        post2Mcu(cmd);
    }

    /**
     * 停止称重校准
     */
    public static void stoptWeightAlign() {
        WeightAlignStopCmd cmd = new WeightAlignStopCmd();
        post2Mcu(cmd);
    }


    /**
     * 开始称重
     */
    public static void startWeight() {
        WeightStartCmd cmd = new WeightStartCmd();
        post2Mcu(cmd);
        devInfo().setWeightStatus(true);
    }

    /**
     * 称重清理
     */
    public static void clearWeight() {
        WeightClearCmd cmd = new WeightClearCmd();
        post2Mcu(cmd);
    }

    /**
     * 停止称重
     */
    public static void stoptWeight() {
        WeightStopCmd cmd = new WeightStopCmd();
        post2Mcu(cmd);
        devInfo().setWeightStatus(false);
    }

    public static void post2Mcu(BaseBody baseBody) {
        if (dev) {
            eventBus.post(Dev2McuDataDevEvent.builder().baseBody(baseBody).build());
        } else {
            eventBus.post(Dev2McuDataEvent.builder().data(baseBody.escapeBytesStr()).build());

        }
    }


    public static void post2Dev(BaseBody baseBody) {
        eventBus.post(Mcu2DevEvent.builder().data(baseBody.escapeBytesStr()).build());
    }


    public static void register(Object subscriber) {
        eventBus.register(subscriber);
    }

    public static void unregister(Object subscriber) {
        eventBus.unregister(subscriber);
    }

    private static int lastErrorTips = -1;

    private static int postErrorTips(int errorCode, boolean focus) {
        if (errorCode == 0) {
            return errorCode;
        }
        if (lastErrorTips == errorCode && !focus) {
            return errorCode;
        }
        lastErrorTips = focus ? -1 : errorCode;


        if (errorCode == 1) { //转速反馈异常
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("zhuan_su_fan_kui_yi_chan")
                    .focus(focus)
                    .build());
        }


        if (errorCode == 2) { //电子秤通信异常
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("dian_zi_cheng_tong_xin_yi_chan")
                    .focus(focus)
                    .build());
        }

//        if (errorCode == 3) { //没放锅
//            eventBus.post(DevPromptEvent.builder()
//                    .showCancelBtn(false)
//                    .msg("mei_fang_guo_yi_chan")
//                    .focus(focus)
//                    .build());
//        }

        if (errorCode == 5) {  //干烧
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("gan_sao_yi_chan")
                    .focus(focus)
                    .build());
        }

        if (errorCode == 6) { //没放锅
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .focus(focus)
                    .msg("ntc_yi_chan")
                    .build());
        }

        return errorCode;
    }


    private static boolean showErrorPotTypeTips = false;

    public static boolean weightOverLoad = false;

    public static void updateDevInfo(DevInfo info) {

        if((System.currentTimeMillis() - lockTime) < 1000){
             return;
        }
        lockTime = System.currentTimeMillis();


        if ((System.currentTimeMillis() - lastDevInfoTime) > 1000) {
            recDevInfo = true;
        }

        if (!recDevInfo) {
            return;
        }
        lastDevInfoTime = System.currentTimeMillis();
        devInfo = info;

        if (info.getRunningStatus() == 0x00) {
            devInfo().setStatus(DevStatus.STOP);
        }

        if (info.getRunningStatus() == 0x01) {
            devInfo().setStatus(DevStatus.RUNNING);
        }
        if (info.getRunningStatus() == 0x02) {
            devInfo().setStatus(DevStatus.PAUSE);
        }

        if (!settingInstId.equals(runningInstId)) {
            DevInfo e = new DevInfo();

            e.setStatus(devInfo.getStatus());
            e.setMotorDirection(settingDevInfo.getMotorDirection());
            e.setMotorDirectionBuffer((byte) -1);
            e.setMotorGear(devInfo.getMotorGear());
            e.setMotorGearBuffer((byte) -1);
            e.setAutoGear(devInfo.getAutoGear());
            e.setHeatMode(devInfo.getHeatMode());
            e.setTemp(devInfo.getTemp());
            e.setTargetTemp(settingDevInfo.getTargetTemp());
            e.setRemainTime((short) 0);
            e.setTargetTime(settingDevInfo.getTargetTime());
            e.setTargetTimeBuffer((short) -1);
            e.setPotStatus(devInfo.getPotStatus());
            e.setPotCloverStatus(devInfo.getPotCloverStatus());
            e.setErrCode(devInfo.getErrCode());
            e.setResetZero(devInfo.getResetZero());
            e.setRunningStatus(devInfo.getRunningStatus());
            e.setPotCloverEleStatus(devInfo.getPotCloverEleStatus());
            e.setWeightAlignStatus(devInfo.getWeightAlignStatus());
//            e.setWeightStatus(devInfo.isWeightStatus());
            e.setWeight(devInfo.getWeight());
            e.setWeightAlignStatus(devInfo.getWeightPol());
            e.setManufacturer(devInfo.getManufacturer());
            e.setHaradware(devInfo.getHaradware());
            e.setSoftware(devInfo.getSoftware());
            e.setProtocol(devInfo.getProtocol());
            e.setDevType(devInfo.getDevType());
            e.setRunningInstId(devInfo.getRunningInstId());
            e.setRunningRecipeExt(devInfo.getRunningRecipeExt());
            e.setRunningInstId(devInfo.getRunningInstId());
            e.setSettingInstId(devInfo.getSettingInstId());

            e.setStatus(DevStatus.STOP);
            e.setTargetTime(settingDevInfo.getTargetTime());
            e.setRemainTime((short) 0);
            e.setMotorGear(settingDevInfo.getMotorGear());
            e.setMotorDirection(settingDevInfo.getMotorDirection());
            e.setTargetTemp(settingDevInfo.getTargetTemp());
            e.setRunningInstId(runningInstId);
            e.setSettingInstId(settingInstId);
            e.setMode(settingDevInfo.getMode());
            eventBus.post(DevStatusEvent.builder().devInfo(e).build());
        } else {
            devInfo.setRunningInstId(runningInstId);
            devInfo.setSettingInstId(settingInstId);
            eventBus.post(DevStatusEvent.builder().devInfo(devInfo).build());
        }


        if (devInfo.getWeight() >= 5300) {
            weightOverLoad=true;
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("weight_overload_tips")
                    .build());
        }else{
            if(weightOverLoad){
                eventBus.post(DevPromptEvent.builder()
                        .action("hide")
                        .showCancelBtn(false)
                        .msg("weight_overload_tips")
                        .build());
            }
            weightOverLoad = false;
        }


        postErrorTips(devInfo.getErrCode(), false);



        //当有设置温度，转速不能高于3
        if ((devInfo().getTargetTemp() > BaseConfig.TEMP_OF_LIMIT_MOTOR || devInfo().getTargetTempBuffer() > BaseConfig.TEMP_OF_LIMIT_MOTOR) && (devInfo().getMotorGear() > devInfo().getMode().getMaxMotorWithTemp() || devInfo().getMotorGearBuffer() > devInfo().getMode().getMaxMotorWithTemp())) {
            eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.MAX_MOTOR_GEAR_RATHER_THEN_WITH_TEMP).build());
        }

        //当有设置温度，转速不能高于4,针对菜谱模式
        if ((devInfo().getTargetTemp() > BaseConfig.TEMP_OF_LIMIT_MOTOR || devInfo().getTargetTempBuffer() > BaseConfig.TEMP_OF_LIMIT_MOTOR) && (devInfo().getMotorGear() > 4 || devInfo().getMotorGearBuffer() > 4)) {
            eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.MOTOR_GEAR_RATHER_THEN_4_WITH_TEMP).build());
        }


        //当转速大于7时，时间最大设置为10分钟
        if (devInfo().getMotorGear() >= 7 || devInfo().getMotorGearBuffer() >= 7) {
            if ((devInfo().getTargetTime() > (10 * 60) && devInfo().getTargetTimeBuffer() == -1) || devInfo().getTargetTimeBuffer() > (10 * 60)) {
                eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.MOTOR_GEAR_RATHER_THEN_7).build());
            }
        }

        //当点击方向为反向的时候，点击转速不能大于3
        if (MotorDirections.REVERSE == devInfo().getMotorDirection() || MotorDirections.REVERSE == devInfo().getMotorDirectionBuffer()) {
            if ((devInfo().getMotorGear() > 3 && devInfo().getMotorGearBuffer() == -1) || devInfo().getMotorGearBuffer() > 3) {
                eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.MOTOR_REVERSE_CAN_NOT_RATHER_THEN_3).build());
            }
        }


        if (!devInfo().isInitStatus()) {

            if (devInfo().getPotCloverType() == PotTypes.SOUP) {
                devInfo.setDevMode(DevModes.SOUP);
            }

            if (devInfo().getPotCloverType() == PotTypes.AIR_FRYER) {
                devInfo.setDevMode(DevModes.AIR_FRYER);
            }
        }

        if (PotStatus.UNEXIST == info.getPotStatus() && info.getLastPotStatus() != info.getPotStatus()) {
            devInfo.setLastPotStatus(info.getPotStatus());
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("mei_fang_guo_yi_chan")
                    .focus(true)
                    .build());
        }

        if (PotStatus.UNEXIST == info.getPotStatus() && info.getLastPotStatus() != info.getPotStatus()) {
            devInfo.setLastPotStatus(info.getPotStatus());
            eventBus.post(DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("mei_fang_guo_yi_chan")
                    .focus(true)
                    .build());
        }

        devInfo.setLastPotStatus(info.getPotStatus());

        if (devInfo.isInitStatus() && !((DevModes.SOUP.equals(devInfo().getDevMode()) && devInfo().getPotCloverType() == PotTypes.SOUP) || (DevModes.AIR_FRYER.equals(devInfo().getDevMode()) && devInfo().getPotCloverType() == PotTypes.AIR_FRYER))) {
            DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                    .showCancelBtn(false)
                    .msg("error_pot_clover_tips")
                    .confirm(() -> {
                        if (devInfo().getPotCloverType() == PotTypes.SOUP) {
                            changeDevModeFocus(DevModes.SOUP);
                        }

                        if (devInfo().getPotCloverType() == PotTypes.AIR_FRYER) {
                            changeDevModeFocus(DevModes.AIR_FRYER);
                        }
                    })
                    .build();
            eventBus.post(devPromptEvent);
        }


//        if (devInfo().getStatus() == DevStatus.STOP || devInfo().getStatus() == DevStatus.RUNNING) {
//             if(devInfo().getTemp() < 60){
//                 switchRedLed('0');
//                 switchWhiteLed('1');
//             }else{
//                 switchRedLed('1');
//                 switchWhiteLed('0');
//             }
//        }
//
//        if (devInfo().getStatus() == DevStatus.PAUSE) {
//            if(devInfo().getTemp() < 60){
//                switchRedLed('0');
//                flashWhiteLed();
//            }else{
//                flashRedLed();
//                switchWhiteLed('0');
//            }
//        }

        if (!devInfo.isInitStatus()) {
            devInfo().setInitStatus(true);
        }

    }

    public static DevInfo devInfo() {
        return devInfo;
    }

    public static short getTempInterval() {
        return 5;
    }

    public static short parseTempNative(short temp) {

        int c = temp / getTempInterval();
        int y = temp % getTempInterval();
        if (y > 2) {
            c = (c + 1);
        }
        return (short) (c * getTempInterval());
    }

    public static short parseTemp(short temp) {

        short t = parseTempNative(temp);

        return t == 35 ? 37 : t;
    }

    public static void init(String brandNum){
        BRAND_NUM = brandNum;
    }

    public static int parseWeight(int weight) {
        return weight;
//        int c = weight / 5;
//        int y = weight % 5;
//        if(y > 2){
//            c=(c+1);
//        }
//        return (c*5);
    }

    public static void updateWeightData(short weight) {
        devInfo().setWeight(weight);
//        devInfo().setWeightStatus(true);
        eventBus.post(DevStatusEvent.builder().devInfo(devInfo).build());

    }

    public static void updateScene(String scene) {
        devInfo().setCurrentScene(scene);
        eventBus.post(DevStatusEvent.builder().devInfo(devInfo).build());
    }

    public static void updateWeightAlignData(byte weightAlignStatus, byte weightPol) {
        devInfo().setWeightAlignStatus(weightAlignStatus);
        devInfo().setWeightPol(weightPol);
        eventBus.post(DevStatusEvent.builder().devInfo(devInfo).build());
    }

    public static void updateVersionInfo(short manufacturer, short haradware, short software, byte protocol, byte devType) {
        devInfo().setManufacturer(manufacturer);
        devInfo().setHaradware(haradware);
        devInfo().setSoftware(software);
        devInfo().setProtocol(protocol);
        devInfo().setDevType(devType);
        eventBus.post(DevStatusEvent.builder().devInfo(devInfo).build());
    }

    public static void postEndOfRunEvent() {
        eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.AUTO_STOP).build());
    }

    public static void postTheConfrimStopEvent() {
        eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.CONFIRM_STOP).build());
    }

    public static void promptUp(DevPromptEvent event) {
        eventBus.post(event);
    }


    public static BaseDevMode devMode(String name,String brandNum){

        if("010".equals(brandNum)){  //针对010客户模式
            if (WorkModes.FERMENT.equals(name)) {
                return new Ferment010D();
            }

            if (WorkModes.SLOW_COOKING.equals(name) || WorkModes.SLOW_COOK.equals(name)) {
                return new SlowCooking010D();
            }
        }

        return baseDevMode(name);
    }

    public static BaseDevMode devMode(String name){

        if(StringUtils.isNotEmpty(BRAND_NUM)){
            return devMode(name,BRAND_NUM);
        }


        return baseDevMode(name);
    }

    /**
     * 初始化模式名
     *
     * @param name 模式名
     * @return
     */
    public static BaseDevMode baseDevMode(String name) {
        if (WorkModes.ADAPTED_COOKING.equals(name)) {
            return new AdaptedCooking();
        }

        if (WorkModes.KETTLE.equals(name) || WorkModes.BOIL_WATER.equals(name)) {
            return new Kettle();
        }


        if (WorkModes.FERMENT.equals(name)) {
            return new Ferment();
        }

        if (WorkModes.CHOP.equals(name)) {
            return new Chop();
        }

        if (WorkModes.TURBO.equals(name)) {
            return new Turbo();
        }

        if (WorkModes.FOOD_PROCESSOR.equals(name)) {
            return new FoodProcessor();
        }

        if (WorkModes.KNEAD.equals(name)) {
            return new Knead();
        }

        if (WorkModes.KNEAD_TOUGH.equals(name)) {
            return new Knead();
        }

        if (WorkModes.STEAM.equals(name)) {
            return new Steam();
        }

        if (WorkModes.SLOW_COOKING.equals(name)) {
            return new SlowCooking();
        }

        if (WorkModes.SLOW_COOK.equals(name)) {
            return new SlowCooking();
        }

        if (WorkModes.COOK_EGGS.equals(name)) {
            return new CookEggs();
        }


        if (WorkModes.COOK_RICE.equals(name)) {
            return new CookRice();
        }

        if (WorkModes.SOVS_VIDE.equals(name)) {
            return new SovsVide();
        }

        if (WorkModes.ROAST.equals(name)) {
            return new Roast();
        }

        if (WorkModes.PUREE.equals(name)) {
            return new Puree();
        }

        if (WorkModes.SMOOTHIE.equals(name)) {
            return new Smoothie();
        }

        if (WorkModes.MULSIFY.equals(name)) {
            return new Mulsify();
        }

        if (WorkModes.AUTO_CLEAN.equals(name)) {
            return new AutoClean();
        }

        if (WorkModes.WEIGHT.equals(name)) {
            return new Weight();
        }


        if (WorkModes.AF_DIY.equals(name)) {
            return new AfDiy();
        }

        if (WorkModes.AF_AIR_FRYER.equals(name)) {
            return new AfAirFryer();
        }

        if (WorkModes.AF_BAKE.equals(name)) {
            return new AfBake();
        }

        if (WorkModes.AF_DRUMSTICKS.equals(name)) {
            return new AfDrumsticks();
        }


        if (WorkModes.AF_EGG_TART.equals(name)) {
            return new AfEggTart();
        }

        if (WorkModes.AF_PIZZA.equals(name)) {
            return new AfPizza();
        }

        if (WorkModes.AF_DEHYDRATE.equals(name)) {
            return new AfDehydrate();
        }

        if (WorkModes.AF_REHEAT.equals(name)) {
            return new AfReheat();
        }

        if (WorkModes.AF_CHICKEN_WING.equals(name)) {
            return new AfChickenWing();
        }

        if (WorkModes.AF_STEAK.equals(name)) {
            return new AfSteak();
        }


        if (WorkModes.AF_SEA_FOOD.equals(name)) {
            return new AfSeaFood();
        }

        if (WorkModes.AF_VEGETABLE.equals(name)) {
            return new AfVegctable();
        }

        if (WorkModes.AF_MUTTON_CHOP.equals(name)) {
            return new AfMuttonChop();
        }



        return DevModes.SOUP.equals(devInfo().getDevMode()) ? new AdaptedCooking() : new AfDiy();

    }


    public static void switchRedLed(char c) {
        try {
            FileOutputStream outputStream = new FileOutputStream("/sys/devices/platform/codeing-key/led2d2_switch");
            outputStream.write(c);
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void switchWhiteLed(char c) {
        try {
            FileOutputStream outputStream = new FileOutputStream("/sys/devices/platform/codeing-key/led2d3_switch");
            outputStream.write(c);
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public static void flashRedLed() {

        try {
            FileInputStream inputStream = new FileInputStream("/sys/devices/platform/codeing-key/led2d2_switch");
            int c = inputStream.read();
            FileOutputStream outputStream = new FileOutputStream("/sys/devices/platform/codeing-key/led2d2_switch");
            outputStream.write(c == '0' ? '1' : '0');
            outputStream.close();
            inputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void flashWhiteRedLed() {

        try {
            FileInputStream inputStream = new FileInputStream("/sys/devices/platform/codeing-key/led2d2_switch");
            int c = inputStream.read();
            FileOutputStream outputStream = new FileOutputStream("/sys/devices/platform/codeing-key/led2d2_switch");
            outputStream.write(c == '0' ? '1' : '0');
            outputStream.close();
            FileOutputStream outputStream2 = new FileOutputStream("/sys/devices/platform/codeing-key/led2d3_switch");
            outputStream2.write(c != '0' ? '1' : '0');
            outputStream.close();
            inputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }


    }

    public static void flashWhiteLed() {

        try {
            FileInputStream inputStream = new FileInputStream("/sys/devices/platform/codeing-key/led2d3_switch");
            int c = inputStream.read();
            FileOutputStream outputStream = new FileOutputStream("/sys/devices/platform/codeing-key/led2d3_switch");
            outputStream.write(c == '0' ? '1' : '0');
            outputStream.close();
            inputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    public static void configBrightness(int num) {

        try {
            FileOutputStream outputStream = new FileOutputStream("/sys/class/backlight/rk28_bl/brightness");
            outputStream.write(num);
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private static Map<String, DevInfo> configRecord = new HashMap<>();

    public static void recordConfig() {
        try {
            DevInfo config = new DevInfo();

            config.setTargetTemp(devInfo.getTargetTemp());
            config.setTargetTime(devInfo.getTargetTime());
            config.setMotorGear(devInfo.getMotorGear());
//            config.setMotorGearBuffer(devInfo.getMotorGearBuffer());
            config.setMotorDirection(devInfo.getMotorDirection());
            configRecord.put(devInfo().getMode().getClass().getName(), config);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public static void changeDevModeFocus(String devMode) {
        devInfo.setDevMode(devMode);
        eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.DEV_MODE_CHANGE).build());

    }

    public static void changeDevMode(String devMode) {
        DevPromptEvent devPromptEvent = DevPromptEvent.builder()
                .msg("confirm_change_dev_mode")
                .confirmButtonText("yes")
                .cancelBtnText("no")
                .showCancelBtn(true)
                .showConfirmBtn(true)
                .confirm(() -> {
                    devInfo.setDevMode(devMode);
                    eventBus.post(DevCommonEvent.builder().type(CommonEventTypes.DEV_MODE_CHANGE).build());

                })
                .build();
        eventBus.post(devPromptEvent);

    }

    public static void main(String[] args) throws Exception {

//        CofarSDK.register(new Mcu2DevEventListener());
//        CofarSDK.register(new Dev2McuEventListener(System.out::println));
//
//        CofarSDK.cfgHeat((short) 5, HeatModes.PU_TONG);
//        CofarSDK.start();
        BaseDevMode dev = new AdaptedCooking();
        System.out.println(dev.getClass().getName());
        System.out.println(481 % 480);

    }





}
